// Google Maps API with Cordova: https://gist.github.com/nikhilJain17/2b82b50b67a44e2e4da4

// Inspiration: https://leafletjs.com/examples/quick-start/
class OpenStreetMap_API {
    static _Franck() {
        return L.icon({
            iconUrl: './img/Franck.jpg',
            iconSize: [20, 24],
            iconAnchor: [10, 12],
            popupAnchor: [-10, -12]
//            shadowUrl: 'my-icon-shadow.png',
//            shadowSize: [20, 24],
//            shadowAnchor: [-10, -12]
        });
    }
    static _Levallois_Perret() {
        return {
            latitude: 48.893217,
            longitude: 2.287864,
            name: "Levallois-Perret"
        };
    }
    static _Paris() {
        return {
            latitude: 48.8534,
            longitude: 2.3488,
            name: "Paris"
        };
    }
    constructor(latitude = OpenStreetMap_API._Levallois_Perret().latitude, longitude = OpenStreetMap_API._Levallois_Perret().longitude) {
        this._latitude = latitude;
        this._longitude = longitude;
        this._map = window.document.getElementById('map'); // Get the display area (DOM must be ready)...
        this._map.style.height = window.screen.height + "px"; // Map *MUST* have height to be visible...
        this._map.style.width = "100%";
        this._options = {// 'PositionOptions'
            timeout: 5000, // 'Infinity' (default): 'getCurrentPosition' won't return until the position is available...
            maximumAge: 1000, // '0' (default): the device cannot use a cached position; it must retrieve the real current position...
            enableHighAccuracy: true // Longer...
        };
        this._map_ = L.map('map' /* 'map' id. within DOM */).setView([this._latitude, this._longitude], /* zoom */ 10);
        // Leaflet ne récupère pas les cartes ("tiles") sur un serveur par défaut.
        // Il faut préciser où les récupérer, e.g., "openstreetmap.fr":
        L.tileLayer('https://{s}.tile.openstreetmap.fr/osmfr/{z}/{x}/{y}.png', {
            // Il est toujours bien de laisser le lien vers la source des données
            attribution: 'data © <a href="//osm.org/copyright">OpenStreetMap</a>/ODbL - rendu <a href="//openstreetmap.fr">OSM France</a>',
            minZoom: 1,
            maxZoom: 20
        }).addTo(this._map_);

        const marker = L.marker([OpenStreetMap_API._Paris().latitude, OpenStreetMap_API._Paris().longitude]).addTo(this._map_);
        marker.bindPopup(OpenStreetMap_API._Paris().name);

        const circle = L.circle([OpenStreetMap_API._Levallois_Perret().latitude, OpenStreetMap_API._Levallois_Perret().longitude], {
            color: 'red',
            fillColor: '#f03',
            fillOpacity: 0.25,
            radius: 500
        }).addTo(this._map_);
        circle.bindPopup(OpenStreetMap_API._Levallois_Perret().name);

        if (window.navigator.geolocation !== null) // 'Geolocation' interface
            // Firefox problem on Mac: https://ruk.ca/content/geolocation-error-user-denied-geolocation-prompt
            this._geolocation_service = window.navigator.geolocation.watchPosition(// Run the geolocation facility...
                    this._geolocation.bind(this), // Success...
                    (failure) => this._inform(failure.code + ": " + failure.message),
                    this._options);
        else
            this._inform("'window.navigator.geolocation === null'");
    }

    _inform(message, timer = 3000) {
        let timer_interval;
        Swal.fire({
            title: message,
            html: 'Bye in <b></b> ms...',
            timer: timer,
            timerProgressBar: true,
            onBeforeOpen: () => {
                Swal.showLoading();
                timer_interval = window.setInterval(() => {
                    const content = Swal.getContent();
                    if (content) {
                        const b = content.querySelector('b');
                        if (b) {
                            b.textContent = Swal.getTimerLeft();
                        }
                    }
                }, 500);
            },
            onClose: () => {
                window.clearInterval(timer_interval);
            }
        }).then((result) => {
            /* Read more about handling dismissals below */
            if (result.dismiss === Swal.DismissReason.timer) {
                window.console.log('I was closed by the timer');
            }
        });
    }

    _geolocation(position /*: GeolocationPosition*/) { // https://developer.mozilla.org/en-US/docs/Web/API/GeolocationPosition
        // https://developer.mozilla.org/en-US/docs/Web/API/GeolocationCoordinates:
        this._map_.setView([position.coords.latitude, position.coords.longitude], /* zoom */ 10);
        const marker = L.marker([position.coords.latitude, position.coords.longitude], {icon: OpenStreetMap_API._Franck()}).addTo(this._map_);
        marker.bindPopup("Franck").openPopup();

        const popup = L.popup();
        this._map_.on('click', event => { // Display latitude and longitude...
            popup.setLatLng(event.latlng).setContent(event.latlng.toString()).openOn(this._map_);
        });
// Closing geolocation facility:
        window.navigator.geolocation.clearWatch(this._geolocation_service);
        this._inform("Geolocation service is stopped...", 1000);
    }
}

const app = {
    initialize: function () {
        window.document.addEventListener('deviceready', this.device_ready.bind(this), false);
//        window.document.addEventListener('pause', this.pause.bind(this), false);
//        window.document.addEventListener('resume', this.resume.bind(this), false);
    },
    device_ready: function () { // 'deviceready' event handler...
        new OpenStreetMap_API();
        this.event_received('deviceready');
    },
    // Update DOM on a Received Event
    event_received: function (id) {
        window.document.getElementById(id).querySelector('.listening').setAttribute('style', 'display:none;');
        window.document.getElementById(id).querySelector('.received').setAttribute('style', 'display:block;');
    }
};

app.initialize();

window.document.onreadystatechange = function () {
    if (window.document.readyState === "interactive")
        // Testing in desktop browser:
        window.setTimeout(() => {
            app.device_ready();
        }, 500);
    else {
        // ...
    }
};

